==> flutter_charts-0.4.0.tar.gz: lib/src/chart/presenter.dart <==
import 'dart:ui' as ui show Paint, PaintingStyle;

import 'options.dart';
import 'container.dart';
import '../util/collection.dart' as custom_collection show CustomList;

// todo-1 refactor - can this be a behavior?
ui.Paint gridLinesPaint(ChartOptions options) {
  ui.Paint paint = ui.Paint();
  paint.color = options.dataContainerOptions.gridLinesColor;
  paint.style = ui.PaintingStyle.stroke;
  paint.strokeWidth = 1.0;

  return paint;
}

/// The visual element representing one data value on the chart.
///
/// Presenter of the atomic/leaf element of one data point on the chart.
///
/// For example, on a bar chart, this is one rectangle;
/// on a line chart this is a point with line connecting to the next
/// value point.
class Presenter {
  // Not actually stored here, but could be
  // todo-13-unused
  StackableValuePoint point;
  StackableValuePoint? nextRightColumnValuePoint;
  int rowIndex;

  Presenter({
    required this.point,
    this.nextRightColumnValuePoint,
    required this.rowIndex,
    required ChartTopContainer chartTopContainer,
  });
}

/// Manages and presents one "visual column" on the chart.
///
/// By one "visual column" here we mean the area above one label, which
/// shows all data value at that label, each value in one instance of
/// [Presenter].
class PresentersColumn {
  List<Presenter> presenters = List.empty(growable: true);
  List<Presenter> positivePresenters = List.empty(growable: true);
  List<Presenter> negativePresenters = List.empty(growable: true);
  PresentersColumn? nextRightPointsColumn;

  PresentersColumn({
    required PointsColumn pointsColumn,
    required ChartTopContainer chartTopContainer,
    required PresenterCreator presenterCreator,
  }) {
    // setup the contained presenters from points
    _createPresentersInColumn(
        fromPoints: pointsColumn.stackableValuePoints,
        toPresenters: presenters,
        pointsColumn: pointsColumn,
        presenterCreator: presenterCreator,
        chartTopContainer: chartTopContainer);
    _createPresentersInColumn(
        fromPoints: pointsColumn.stackedPositivePoints,
        toPresenters: positivePresenters,
        pointsColumn: pointsColumn,
        presenterCreator: presenterCreator,
        chartTopContainer: chartTopContainer);
    _createPresentersInColumn(
        fromPoints: pointsColumn.stackedNegativePoints,
        toPresenters: negativePresenters,
        pointsColumn: pointsColumn,
        presenterCreator: presenterCreator,
        chartTopContainer: chartTopContainer);
  }

  void _createPresentersInColumn({
    required List<StackableValuePoint> fromPoints,
    required List toPresenters,
    required PointsColumn pointsColumn,
    required PresenterCreator presenterCreator,
    required ChartTopContainer chartTopContainer,
  }) {
    int rowIndex = 0;
    for (StackableValuePoint point in fromPoints) {
      // todo-11-later nextRightPointsColumn IS LIKELY UNUSED, REMOVE.
      StackableValuePoint? nextRightColumnValuePoint = pointsColumn.nextRightPointsColumn != null
          ? pointsColumn.nextRightPointsColumn!.stackableValuePoints[rowIndex]
          : null;

      Presenter presenter = presenterCreator.createPointPresenter(
        point: point,
        nextRightColumnValuePoint: nextRightColumnValuePoint,
        rowIndex: point.dataRowIndex,
        chartTopContainer: chartTopContainer,
      );
      toPresenters.add(presenter);
      rowIndex++;
    }
  }
}

/// Manages the visual elements (leafs) presented in each
/// "visual column" in chart - that is, all widgets representing
/// series of data displayed above each X label.
///
/// The "column oriented" list of data is managed by [PointsColumns.pointsColumns],
/// and is a "source" for creating this object.
/// In addition to [PointsColumns.pointsColumns], a constructor
/// of this object needs to be given a way to create each "visual atomic widget"
/// to display each data value. This is provided by the passed
/// [PresenterCreator], which "create" methods know how to create the concrete
/// instances of the "atomic stacked display widget of the data value" using
///
///   [PresenterCreator.createPointPresenter]
///
/// Notes:
///   - Each [PresentersColumn] element of [presentersColumns]
///     manages a link to the [PresentersColumn] on it's right, allowing
///     walk without the [presentersColumns] list.
class PresentersColumns extends custom_collection.CustomList<PresentersColumn> {
  PresentersColumns({
    required PointsColumns pointsColumns,
    required ChartTopContainer chartTopContainer,
    required PresenterCreator presenterCreator,
  }) {
    // iterate "column oriented", that is, over valuePointsColumns.
    PresentersColumn? leftPresentersColumn;
    for (PointsColumn pointsColumn in pointsColumns) {
      var presentersColumn = PresentersColumn(
        pointsColumn: pointsColumn,
        chartTopContainer: chartTopContainer,
        presenterCreator: presenterCreator,
      );
      add(presentersColumn);
      leftPresentersColumn?.nextRightPointsColumn = presentersColumn;
      leftPresentersColumn = presentersColumn;
    }
  }
}

/// Maker of [Presenter] instances.
///
/// It's core method [createPointPresenter] creates [Presenter]s,
/// the visuals painted on each chart column that
/// represent data, (points and lines for the line chart,
/// rectangles for the bar chart, and so on).
///
/// The concrete creators make [LineAndHotspotPresenter], [VerticalBarPresenter]
/// and other concrete instances, depending on the chart type.
abstract class PresenterCreator {
  /// The container is generally needed for the creation of Presenters, as
  /// presenters may need some layout values.
  PresenterCreator(); // same as  {}

  Presenter createPointPresenter({
    required StackableValuePoint point,
    StackableValuePoint? nextRightColumnValuePoint,
    required int rowIndex,
    required ChartTopContainer chartTopContainer,
  });
}

==> flutter_charts-0.5.0.tar.gz: lib/src/chart/presenter.dart <==
import 'dart:ui' as ui show Paint, PaintingStyle;

import 'options.dart';
import 'container.dart';
import '../util/collection.dart' as custom_collection show CustomList;

// todo-1 refactor - can this be a behavior?
ui.Paint gridLinesPaint(ChartOptions options) {
  ui.Paint paint = ui.Paint();
  paint.color = options.dataContainerOptions.gridLinesColor;
  paint.style = ui.PaintingStyle.stroke;
  paint.strokeWidth = 1.0;

  return paint;
}

/// The visual element representing one data value on the chart.
///
/// Presenter of the atomic/leaf element of one data point on the chart.
///
/// For example, on a bar chart, this is one rectangle;
/// on a line chart this is a point with line connecting to the next
/// value point.
class Presenter {
  // Not actually stored here, but could be
  // todo-01-morph-unused
  StackableValuePoint point;
  StackableValuePoint? nextRightColumnValuePoint;
  int rowIndex;

  Presenter({
    required this.point,
    this.nextRightColumnValuePoint,
    required this.rowIndex,
    required ChartTopContainer chartTopContainer,
  });
}

/// Manages and presents one "visual column" on the chart.
///
/// By one "visual column" here we mean the area above one label, which
/// shows all data value at that label, each value in one instance of
/// [Presenter].
class PresentersColumn {
  List<Presenter> presenters = List.empty(growable: true);
  List<Presenter> positivePresenters = List.empty(growable: true);
  List<Presenter> negativePresenters = List.empty(growable: true);
  PresentersColumn? nextRightPointsColumn;

  PresentersColumn({
    required PointsColumn pointsColumn,
    required ChartTopContainer chartTopContainer,
    required PresenterCreator presenterCreator,
  }) {
    // setup the contained presenters from points
    _createPresentersInColumn(
        fromPoints: pointsColumn.stackableValuePoints,
        toPresenters: presenters,
        pointsColumn: pointsColumn,
        presenterCreator: presenterCreator,
        chartTopContainer: chartTopContainer);
    _createPresentersInColumn(
        fromPoints: pointsColumn.stackedPositivePoints,
        toPresenters: positivePresenters,
        pointsColumn: pointsColumn,
        presenterCreator: presenterCreator,
        chartTopContainer: chartTopContainer);
    _createPresentersInColumn(
        fromPoints: pointsColumn.stackedNegativePoints,
        toPresenters: negativePresenters,
        pointsColumn: pointsColumn,
        presenterCreator: presenterCreator,
        chartTopContainer: chartTopContainer);
  }

  void _createPresentersInColumn({
    required List<StackableValuePoint> fromPoints,
    required List toPresenters,
    required PointsColumn pointsColumn,
    required PresenterCreator presenterCreator,
    required ChartTopContainer chartTopContainer,
  }) {
    int rowIndex = 0;
    for (StackableValuePoint point in fromPoints) {
      // todo-01-last nextRightPointsColumn IS LIKELY UNUSED, REMOVE.
      StackableValuePoint? nextRightColumnValuePoint = pointsColumn.nextRightPointsColumn != null
          ? pointsColumn.nextRightPointsColumn!.stackableValuePoints[rowIndex]
          : null;

      Presenter presenter = presenterCreator.createPointPresenter(
        point: point,
        nextRightColumnValuePoint: nextRightColumnValuePoint,
        rowIndex: point.dataRowIndex,
        chartTopContainer: chartTopContainer,
      );
      toPresenters.add(presenter);
      rowIndex++;
    }
  }
}

/// Manages the visual elements (leafs) presented in each
/// "visual column" in chart - that is, all widgets representing
/// series of data displayed above each X label.
///
/// The "column oriented" list of data is managed by [PointsColumns.pointsColumns],
/// and is a "source" for creating this object.
/// In addition to [PointsColumns.pointsColumns], a constructor
/// of this object needs to be given a way to create each "visual atomic widget"
/// to display each data value. This is provided by the passed
/// [PresenterCreator], which "create" methods know how to create the concrete
/// instances of the "atomic stacked display widget of the data value" using
///
///   [PresenterCreator.createPointPresenter]
///
/// Notes:
///   - Each [PresentersColumn] element of [presentersColumns]
///     manages a link to the [PresentersColumn] on it's right, allowing
///     walk without the [presentersColumns] list.
class PresentersColumns extends custom_collection.CustomList<PresentersColumn> {
  PresentersColumns({
    required PointsColumns pointsColumns,
    required ChartTopContainer chartTopContainer,
    required PresenterCreator presenterCreator,
  }) {
    // iterate "column oriented", that is, over valuePointsColumns.
    PresentersColumn? leftPresentersColumn;
    for (PointsColumn pointsColumn in pointsColumns) {
      var presentersColumn = PresentersColumn(
        pointsColumn: pointsColumn,
        chartTopContainer: chartTopContainer,
        presenterCreator: presenterCreator,
      );
      add(presentersColumn);
      leftPresentersColumn?.nextRightPointsColumn = presentersColumn;
      leftPresentersColumn = presentersColumn;
    }
  }
}

/// Maker of [Presenter] instances.
///
/// It's core method [createPointPresenter] creates [Presenter]s,
/// the visuals painted on each chart column that
/// represent data, (points and lines for the line chart,
/// rectangles for the bar chart, and so on).
///
/// The concrete creators make [LineAndHotspotPresenter], [VerticalBarPresenter]
/// and other concrete instances, depending on the chart type.
abstract class PresenterCreator {
  /// The container is generally needed for the creation of Presenters, as
  /// presenters may need some layout values.
  PresenterCreator(); // same as  {}

  Presenter createPointPresenter({
    required StackableValuePoint point,
    StackableValuePoint? nextRightColumnValuePoint,
    required int rowIndex,
    required ChartTopContainer chartTopContainer,
  });
}
